@@ -4,21 +4,21 @@
 
 set -ev
 rm -rf temp1
-$DARCS init --repodir temp1
+darcs init --repodir temp1
 
 export EMAIL=me
 
 cd temp1
 
 date > f
-$DARCS add f
-$DARCS rec -am p1
+darcs add f
+darcs rec -am p1
 for (( i=0 ; i < 500; i=i+1 )); do
   echo $i >> f;
-  $DARCS rec -am p$i
+  darcs rec -am p$i
 done
 
-$DARCS changes 2> err | head
+darcs changes 2> err | head
 
 touch correcterr
 
