@@ -1,4 +1,4 @@
-{-# LANGUAGE BangPatterns, ForeignFunctionInterface, CPP #-}
+{-# LANGUAGE BangPatterns, ForeignFunctionInterface, CPP, ScopedTypeVariables #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -26,6 +26,12 @@ module ByteStringUtils (
         gzWriteFilePS,
         gzWriteFilePSs,
 
+        -- gzip handling
+        isGZFile,
+#ifdef HAVE_HASKELL_ZLIB
+        gzDecompress,
+#endif
+
         -- list utilities
         ifHeadThenTail,
         dropSpace,
@@ -46,30 +52,23 @@ module ByteStringUtils (
         intercalate
     ) where
 
-import Autoconf                 ( use_mmap )
-
+import Prelude hiding ( catch )
 import qualified Data.ByteString            as B
 import qualified Data.ByteString.Char8      as BC
-#if __GLASGOW_HASKELL__ > 606
 import qualified Data.ByteString.Internal   as BI
 import Data.ByteString (intercalate, uncons)
 import Data.ByteString.Internal (fromForeignPtr)
-#else
-import qualified Data.ByteString.Base     as BI
-#endif
 
+#if defined (HAVE_MMAP) || ! defined (HAVE_HASKELL_ZLIB)
+import Control.Exception        ( catch )
+#endif
 import System.IO
 import System.IO.Unsafe         ( unsafePerformIO )
 
-#if __GLASGOW_HASKELL__ > 606
 import Foreign.Storable         ( peekElemOff, peek )
-#else
-import Foreign.Storable         ( peekElemOff, peek, peekByteOff )
-import Data.List                ( intersperse )
-#endif
 import Foreign.Marshal.Alloc    ( free )
 import Foreign.Marshal.Array    ( mallocArray, peekArray, advancePtr )
-import Foreign.C.Types          ( CInt, CSize )
+import Foreign.C.Types          ( CInt )
 
 import Data.Bits                ( rotateL )
 import Data.Char                ( chr, ord, isSpace )
@@ -77,13 +76,12 @@ import Data.Word                ( Word8 )
 import Data.Int                 ( Int32 )
 import Control.Monad            ( when )
 
-import Foreign.Ptr              ( nullPtr, plusPtr, Ptr )
-import Foreign.ForeignPtr       ( ForeignPtr, withForeignPtr )
-
-#if defined(__GLASGOW_HASKELL__)
-import qualified Foreign.Concurrent as FC ( newForeignPtr )
-import System.Posix             ( handleToFd )
+#ifndef HAVE_HASKELL_ZLIB
+import Foreign.Ptr              ( nullPtr )
+import Foreign.ForeignPtr       ( ForeignPtr )
 #endif
+import Foreign.Ptr              ( plusPtr, Ptr )
+import Foreign.ForeignPtr       ( withForeignPtr )
 
 #ifdef DEBUG_PS
 import Foreign.ForeignPtr       ( addForeignPtrFinalizer )
@@ -93,13 +91,22 @@ import Foreign.Ptr              ( FunPtr )
 #if HAVE_HASKELL_ZLIB
 import qualified Data.ByteString.Lazy as BL
 import qualified Codec.Compression.GZip as GZ
+import qualified Codec.Compression.Zlib.Internal as ZI
+import Darcs.Global ( addCRCWarning )
 #else
 import Foreign.C.String ( CString, withCString )
 #endif
 
+#ifdef HAVE_MMAP
+import System.IO.MMap( mmapFileByteString )
+import System.Mem( performGC )
+import System.Posix.Files( fileSize, getSymbolicLinkStatus )
+#endif
+
 -- -----------------------------------------------------------------------------
 -- obsolete debugging code
 
+# ifndef HAVE_HASKELL_ZLIB
 debugForeignPtr :: ForeignPtr a -> String -> IO ()
 #ifdef DEBUG_PS
 foreign import ccall unsafe "static fpstring.h debug_alloc" debug_alloc
@@ -113,6 +120,7 @@ debugForeignPtr fp n =
 #else
 debugForeignPtr _ _ = return ()
 #endif
+#endif
 
 -- -----------------------------------------------------------------------------
 -- unsafeWithInternals
@@ -162,32 +170,6 @@ ifHeadThenTail c s = case uncons s of
     Just (w, t) | w == c    -> Just t
     _                       -> Nothing
 
-#if __GLASGOW_HASKELL__ <= 606
--- | /O(1)/ Extract the head and tail of a ByteString, returning Nothing
--- if it is empty.
-uncons :: B.ByteString -> Maybe (Word8, B.ByteString)
-uncons (BI.PS x s l)
-    | l <= 0    = Nothing
-    | otherwise = Just (BI.inlinePerformIO $ withForeignPtr x
-                                        $ \p -> peekByteOff p s,
-                        BI.PS x (s+1) (l-1))
-{-# INLINE uncons #-}
--- | /O(1)/ Build a ByteString from a ForeignPtr
-fromForeignPtr :: ForeignPtr Word8
-               -> Int -- ^ Offset
-               -> Int -- ^ Length
-               -> B.ByteString
-fromForeignPtr fp s l = BI.PS fp s l
-{-# INLINE fromForeignPtr #-}
--- | /O(n)/ The 'intercalate' function takes a 'ByteString' and a list of
--- 'ByteString's and concatenates the list after interspersing the first
--- argument between each element of the list.
-intercalate :: B.ByteString -> [B.ByteString] -> B.ByteString
-intercalate s = B.concat . (intersperse s)
-{-# INLINE [1] intercalate #-}
-#endif
-
-
 ------------------------------------------------------------------------
 -- A reimplementation of Data.ByteString.Char8.dropSpace, but
 -- specialised to darcs' need for a 4 way isspace.
@@ -361,10 +343,43 @@ unlinesPSOld ss = BC.concat $ intersperse_newlines ss
 -- -----------------------------------------------------------------------------
 -- gzReadFilePS
 
--- | Read an entire file, which may or may not be gzip compressed, directly
--- into a 'B.ByteString'.
+#ifdef HAVE_HASKELL_ZLIB
+
+-- |Decompress the given bytestring into a lazy list of chunks, along with a boolean
+-- flag indicating (if True) that the CRC was corrupted.
+-- Inspecting the flag will cause the entire list of chunks to be evaluated (but if
+-- you throw away the list immediately this should run in constant space).
+gzDecompress :: Maybe Int -> BL.ByteString -> ([B.ByteString], Bool)
+gzDecompress mbufsize =
+    -- This is what the code would be without the bad CRC recovery logic:
+    -- return . BL.toChunks . GZ.decompressWith decompressParams
+    toListWarn . ZI.decompressWithErrors ZI.GZip decompressParams
+  where
+        decompressParams = case mbufsize of
+                              Just bufsize -> GZ.defaultDecompressParams { GZ.decompressBufferSize = bufsize }
+                              Nothing -> GZ.defaultDecompressParams
+
+        toListWarn :: ZI.DecompressStream -> ([B.ByteString], Bool)
+        toListWarn = foldDecompressStream (\x ~(xs, b) -> (x:xs, b)) ([], False) handleBad
+
+        -- cut and paste from Zlib since it's not currently exported (interface not yet certain)
+        foldDecompressStream :: (B.ByteString -> a -> a) -> a
+                             -> (ZI.DecompressError -> String -> a)
+                             -> ZI.DecompressStream -> a
+        foldDecompressStream chunk end err = fold
+                   where
+                       fold ZI.StreamEnd               = end
+                       fold (ZI.StreamChunk bs stream) = chunk bs (fold stream)
+                       fold (ZI.StreamError code msg)  = err code msg
+
+        -- For a while a bug in darcs caused gzip files with good data but bad CRCs to be
+        -- produced. Trap bad CRC messages, run the specified action to report that it happened,
+        -- but continue on the assumption that the data is valid.
+        handleBad ZI.DataError "incorrect data check" = ([], True)
+        handleBad _ msg = error msg
+
+#else
 
-#ifndef HAVE_HASKELL_ZLIB
 foreign import ccall unsafe "static zlib.h gzopen" c_gzopen
     :: CString -> CString -> IO (Ptr ())
 foreign import ccall unsafe "static zlib.h gzclose" c_gzclose
@@ -373,36 +388,38 @@ foreign import ccall unsafe "static zlib.h gzread" c_gzread
     :: Ptr () -> Ptr Word8 -> CInt -> IO CInt
 foreign import ccall unsafe "static zlib.h gzwrite" c_gzwrite
     :: Ptr () -> Ptr Word8 -> CInt -> IO CInt
+
 #endif
 
-gzReadFilePS :: FilePath -> IO B.ByteString
-gzReadFilePS f = do
+isGZFile :: FilePath -> IO (Maybe Int)
+isGZFile f = do
     h <- openBinaryFile f ReadMode
     header <- B.hGet h 2
     if header /= BC.pack "\31\139"
        then do hClose h
-               mmapFilePS f
+               return Nothing
        else do hSeek h SeekFromEnd (-4)
                len <- hGetLittleEndInt h
                hClose h
+               return (Just len)
+
+-- | Read an entire file, which may or may not be gzip compressed, directly
+-- into a 'B.ByteString'.
+gzReadFilePS :: FilePath -> IO B.ByteString
+gzReadFilePS f = do
+    mlen <- isGZFile f
+    case mlen of
+       Nothing -> mmapFilePS f
+       Just len ->
 #ifdef HAVE_HASKELL_ZLIB
-               -- Passing the length to GZ.decompressWith means
-               -- that BL.toChunks only produces one chunk, which in turn
-               -- means that B.concat won't need to copy data.
+            do -- Passing the length to gzDecompress means that it produces produces one chunk,
+               -- which in turn means that B.concat won't need to copy data.
                -- If the length is wrong this will just affect efficiency, not correctness
-               let decompress = GZ.decompressWith GZ.defaultDecompressParams {
-                                  GZ.decompressBufferSize = len
-                                }
-               fmap (B.concat . BL.toChunks . decompress) $
-#ifdef HAVE_OLD_BYTESTRING
-                        -- bytestring < 0.9.1 had a bug where it did not know to close handles upon EOF
-                        -- performance would be better with a newer bytestring and lazy
-                        -- readFile below -- ratify readFile: comment
-                        fmap (BL.fromChunks . (:[])) $
-                        B.readFile f  -- ratify readFile: immediately consumed
-#else
-                        BL.readFile f -- ratify readFile: immediately consumed by the conversion to a strict bytestring
-#endif
+               let doDecompress buf = let (res, bad) = gzDecompress (Just len) buf
+                                      in do when bad $ addCRCWarning f
+                                            return res
+               compressed <- (BL.fromChunks . return) `fmap` mmapFilePS f
+               B.concat `fmap` doDecompress compressed
 #else
                withCString f $ \fstr-> withCString "rb" $ \rb-> do
                  gzf <- c_gzopen fstr rb
@@ -466,58 +483,18 @@ gzWriteToGzf gzf ps = case BI.toForeignPtr ps of
 -- the file is assumed to be ISO-8859-1.
 
 mmapFilePS :: FilePath -> IO B.ByteString
-mmapFilePS f = if use_mmap
-               then do (fp,l) <- mmap f
-                       return $ fromForeignPtr fp 0 l
-               else B.readFile f
-
-#if defined(__GLASGOW_HASKELL__)
-foreign import ccall unsafe "static fpstring.h my_mmap" my_mmap
-    :: CSize -> CInt -> IO (Ptr Word8)
-foreign import ccall unsafe "static sys/mman.h munmap" c_munmap
-    :: Ptr Word8 -> CSize -> IO CInt
-foreign import ccall unsafe "static unistd.h close" c_close
-    :: CInt -> IO CInt
-#endif
-
-mmap :: FilePath -> IO (ForeignPtr Word8, Int)
-mmap f = do
-    h <- openBinaryFile f ReadMode
-    l <- fromIntegral `fmap` hFileSize h
-    -- Don't bother mmaping small files because each mmapped file takes up
-    -- at least one full VM block.
-    if l < mmap_limit
-       then do thefp <- BI.mallocByteString l
-               debugForeignPtr thefp $ "mmap short file "++f
-               withForeignPtr thefp $ \p-> hGetBuf h p l
-               hClose h
-               return (thefp, l)
-       else do
-#if defined(__GLASGOW_HASKELL__)
-               fd <- fromIntegral `fmap` handleToFd h
-               p <- my_mmap (fromIntegral l) fd
-               fp <- if p == nullPtr
-                     then
+#ifdef HAVE_MMAP
+mmapFilePS f = do
+  x <- mmapFileByteString f Nothing
+   `catch` (\_ -> do
+                     size <- fileSize `fmap` getSymbolicLinkStatus f
+                     if size == 0
+                        then return B.empty
+                        else performGC >> mmapFileByteString f Nothing)
+  return x
 #else
-               fp <-
+mmapFilePS = B.readFile
 #endif
-                          do thefp <- BI.mallocByteString l
-                             debugForeignPtr thefp $ "mmap short file "++f
-                             withForeignPtr thefp $ \p' -> hGetBuf h p' l
-                             return thefp
-#if defined(__GLASGOW_HASKELL__)
-                     else do
-                             fp <- FC.newForeignPtr p
-                                   (do {c_munmap p $ fromIntegral l;
-                                        return (); })
-                             debugForeignPtr fp $ "mmap "++f
-                             return fp
-               c_close fd
-#endif
-               hClose h
-               return (fp, l)
-    where mmap_limit = 16*1024
-
 
 -- -------------------------------------------------------------------------
 -- fromPS2Hex
