@@ -13,36 +13,36 @@ module Darcs.External (
     maybeURLCmd,
     Cachable(Cachable, Uncachable, MaxAge),
     viewDoc, viewDocWith,
+    sendmail_path, diff_program
   ) where
 
-import Data.List ( intersperse )
-import Control.Monad ( when, zipWithM_ )
+import Data.Maybe ( isJust, isNothing, maybeToList )
+import Control.Monad ( when, zipWithM_, filterM, liftM2 )
 import System.Exit ( ExitCode(..) )
 import System.Environment ( getEnv )
-import System.Cmd ( system )
 import System.IO ( hPutStr, hPutStrLn, hGetContents, hClose,
                    openBinaryFile, IOMode( ReadMode ),
                    openBinaryTempFile,
                    hIsTerminalDevice, stdout, stderr, Handle )
 import System.IO.Error ( isDoesNotExistError )
-import System.IO.Unsafe ( unsafePerformIO )
 import System.Posix.Files ( getSymbolicLinkStatus, isRegularFile, isDirectory )
 import System.Directory ( createDirectory, getDirectoryContents,
                           doesFileExist, doesDirectoryExist,
-                          renameFile, renameDirectory, copyFile )
+                          renameFile, renameDirectory, copyFile,
+                          findExecutable )
 import System.Process ( runProcess, runInteractiveProcess, waitForProcess )
 import Control.Concurrent ( forkIO, newEmptyMVar, putMVar, takeMVar )
 import Control.Exception ( bracket, try, finally )
 import Data.Char ( toUpper )
+#if defined (HAVE_MAPI)
 import Foreign.C ( CString, withCString )
+#endif
+#ifdef HAVE_MAPI
 import Foreign.Ptr ( nullPtr )
+import Darcs.Lock ( canonFilename, writeDocBinFile )
+#endif
 #ifdef HAVE_TERMINFO
 import System.Console.Terminfo( tiGetNum, setupTermFromEnv, getCapability )
-#elif HAVE_CURSES
-import Foreign.C ( CChar, CInt )
-import Foreign.Ptr ( Ptr )
-import Foreign.Marshal.Alloc (allocaBytes)
-import Autoconf ( use_color )
 #endif
 import System.Posix.Files ( createLink )
 import System.Directory ( createDirectoryIfMissing )
@@ -57,23 +57,36 @@ import Progress ( withoutProgress, progressList, debugMessage )
 import ByteStringUtils (gzReadFilePS, linesPS, unlinesPS)
 import qualified Data.ByteString as B (ByteString, empty, null, readFile -- ratify readFile: Just an import from ByteString
             ,hGetContents, writeFile, hPut, length -- ratify hGetContents: importing from ByteString
-            ,take, concat, drop, isPrefixOf)
+            ,take, concat, drop, isPrefixOf, singleton, append)
 import qualified Data.ByteString.Char8 as BC (unpack, pack)
 
-import Darcs.Lock ( withTemp, withOpenTemp, tempdir_loc, canonFilename, writeDocBinFile,
-                    removeFileMayNotExist, )
+import Darcs.Lock ( withTemp, withOpenTemp, tempdir_loc, removeFileMayNotExist )
 import CommandLine ( parseCmd, addUrlencoded )
-import Autoconf ( have_libcurl, have_libwww, have_HTTP, have_sendmail, have_mapi, sendmail_path, darcs_version )
 import URL ( copyUrl, copyUrlFirst, waitUrl )
 import Ssh ( getSSH, copySSH, copySSHs, SSHCmd(..) )
 import URL ( Cachable(..) )
 import Exec ( exec, Redirect(..), withoutNonBlock )
 import Darcs.URL ( is_file, is_url, is_ssh )
 import Darcs.Utils ( catchall )
-import Printer ( Doc, Printers, putDocLnWith, hPutDoc, hPutDocLn, hPutDocWith, ($$), (<+>), renderPS,
+import Printer ( Doc, Printers, putDocLnWith, hPutDoc, hPutDocLn, hPutDocWith, ($$), renderPS,
                  simplePrinters,
                  text, empty, packedString, vcat, renderString )
-#include "impossible.h"
+import Darcs.Email ( formatHeader )
+
+sendmail_path :: IO String
+sendmail_path = do
+  l <- filterM doesFileExist $ liftM2 (</>)
+                [ "/usr/sbin", "/sbin", "/usr/lib" ]
+                [ "sendmail" ]
+  ex <- findExecutable "sendmail"
+  when (isNothing ex && null l) $ fail "Cannot find the \"sendmail\" program."
+  return $ head $ maybeToList ex ++ l
+
+diff_program :: IO String
+diff_program = do
+  l <- filterM (fmap isJust . findExecutable) [ "gdiff", "gnudiff", "diff" ]
+  when (null l) $ fail "Cannot find the \"diff\" program."
+  return $ head l
 
 backupByRenaming :: FilePath -> IO ()
 backupByRenaming = backupBy rename
@@ -105,6 +118,9 @@ backupBy backup f =
              where next = f ++ suffix
                    suffix = "-darcs-backup" ++ show i
 
+-- | @fetchFile fileOrUrl cache@ returns the content of its argument
+-- (either a file or an URL). If it has to download an url, then it
+-- will use a cache as required by its second argument.
 fetchFilePS :: String -> Cachable -> IO B.ByteString
 fetchFilePS fou _ | is_file fou = B.readFile fou
 fetchFilePS fou cache = withTemp $ \t -> do copyFileOrUrl [] fou t cache
@@ -197,13 +213,15 @@ maybeURLCmd what url =
              `catch` \_ -> return Nothing
 
 speculateRemote :: String -> FilePath -> IO () -- speculations are always Cachable
+#if defined(HAVE_CURL) || defined(HAVE_HTTP)
 speculateRemote u v =
     do maybeget <- maybeURLCmd "GET" u
        case maybeget of
          Just _ -> return () -- can't pipeline these
-         Nothing -> if have_libwww || have_libcurl || have_HTTP
-                    then copyUrl u v Cachable
-                    else return ()
+         Nothing -> copyUrl u v Cachable
+#else
+speculateRemote u _ = maybeURLCmd "GET" u >> return ()
+#endif
 
 copyRemote :: String -> FilePath -> Cachable -> IO ()
 copyRemote u v cache =
@@ -217,9 +235,7 @@ copyRemote u v cache =
                   fail $ "(" ++ get ++ ") failed to fetch: " ++ u
 
 copyRemoteNormal :: String -> FilePath -> Cachable -> IO ()
-copyRemoteNormal u v cache = if have_libwww || have_libcurl || have_HTTP
-                             then copyUrlFirst u v cache >> waitUrl u
-                             else copyRemoteCmd u v
+copyRemoteNormal u v cache = copyUrlFirst u v cache >> waitUrl u
 
 copyFilesOrUrls :: [DarcsFlag]->FilePath->[String]->FilePath->Cachable->IO ()
 copyFilesOrUrls opts dou ns out _ | is_file dou = copyLocals opts dou ns out
@@ -269,67 +285,12 @@ mgetRemotes mget u ns d = do
 
 copyRemotesNormal :: String -> [String] -> FilePath -> Cachable -> IO()
 copyRemotesNormal u ns d cache =
-    if have_libwww || have_libcurl || have_HTTP
-    then do mapM_ (\n -> copyUrl (u++"/"++n) (d++"/"++n) cache) ns
-            doWithPatches (\n -> waitUrl (u++"/"++n)) ns
-    else wgetRemotes u ns d
-
--- Argh, this means darcs get will fail if we don't have libcurl and don't
--- have wget.  :(
-wgetRemotes :: String -> [String] -> FilePath -> IO ()
-wgetRemotes u ns d = do wget_command <- getEnv "DARCS_WGET" `catch`
-                                               \_ -> return "wget"
-                        let (wget, wget_args) = breakCommand wget_command
-                            input = unlines $ map (\n -> u++"/"++n) ns
-                        withCurrentDirectory d $ withOpenTemp $ \(th,tn) ->
-                            do hPutStr th input
-                               hClose th
-                               r <- exec wget (wget_args++["-i",tn])
-                                         (Null,Null,AsIs)
-                               when (r /= ExitSuccess) $
-                                    fail $ unlines $
-                                             ["(wget) failed to fetch files.",
-                                              "source directory: " ++ d,
-                                              "source files:"] ++ ns
-
-copyRemoteCmd :: String -> FilePath -> IO ()
-copyRemoteCmd s tmp = do
-    let cmd = get_ext_cmd
-    r <- stupidexec (cmd tmp s) (Null,Null,AsIs)
-    when (r /= ExitSuccess) $
-         fail $ "failed to fetch: " ++ s ++" " ++ show r
-    where stupidexec [] = bug "stupidexec without a command"
-          stupidexec xs = exec (head xs) (tail xs)
+    do mapM_ (\n -> copyUrl (u++"/"++n) (d++"/"++n) cache) ns
+       doWithPatches (\n -> waitUrl (u++"/"++n)) ns
 
 doWithPatches :: (String -> IO ()) -> [String] -> IO ()
 doWithPatches f patches = mapM_ (\p -> seq p $ f p) $ progressList "Copying patch" patches
 
-{-# NOINLINE get_ext_cmd #-}
-get_ext_cmd :: String -> String -> [String]
--- Only need to find the command once..
-get_ext_cmd = unsafePerformIO get_ext_cmd'
-
--- Would be better to read possible command lines from config-file..
-get_ext_cmd' :: IO (String -> String -> [String])
-get_ext_cmd' = try_cmd cmds
-    where cmds = [("wget", (("--version",0),
-                          -- use libcurl for proper cache control
-                          \t s -> ["wget", "-q",
-                                   "--header=Pragma: no-cache",
-                                   "--header=Cache-Control: no-cache",
-                                   "-O",t,s])),
-                  ("curl", (("--version",2),
-                            \t s -> ["curl", "-s", "-f", "-L",
-                                     "-H", "Pragma: no-cache",
-                                     "-H", "Cache-Control: no-cache",
-                                     "-o",t,s]))]
-          try_cmd [] = fail $ "I need one of: " ++ cs
-              where cs = concat $ intersperse ", " (map fst cmds)
-          try_cmd ((c,(ok_check,f)):cs) = do
-            True <- can_execute ok_check c
-            return f
-           `catch` (\_ -> try_cmd cs)
-
 -- | Run a command on a remote location without passing it any input or
 --   reading its output.  Return its ExitCode
 execSSH :: String -> String -> IO ExitCode
@@ -380,17 +341,18 @@ generateEmail
     -> Doc     -- ^ body
     -> IO ()
 generateEmail h f t s cc body = do
-     hPutDocLn h $
-           text "To:"      <+> text t
-        $$ text "From:"    <+> text f
-        $$ text "Subject:" <+> text s
-        $$ formated_cc
-        $$ text "X-Mail-Originator: Darcs Version Control System"
-        $$ text ("X-Darcs-Version: " ++ darcs_version)
-        $$ body
-  where formated_cc = if cc == ""
-                      then empty
-                      else text "Cc:" <+> text cc
+     putHeader "To" t
+     putHeader "From" f
+     putHeader "Subject" s
+     when (not (null cc)) (putHeader "Cc" cc)
+     putHeader "X-Mail-Originator" "Darcs Version Control System"
+     hPutDocLn h body
+  where putHeader field value
+            = B.hPut h (B.append (formatHeader field value) newline)
+        newline = B.singleton 10
+
+have_sendmail :: IO Bool
+have_sendmail = (sendmail_path >> return True) `catch` (\_ -> return False)
 
 -- | Send an email, optionally containing a patch bundle
 --   (more precisely, its description and the bundle itself)
@@ -406,8 +368,9 @@ sendEmailDoc
 sendEmailDoc _ "" _ "" _ _ _ = return ()
 sendEmailDoc f "" s cc scmd mbundle body =
   sendEmailDoc f cc s "" scmd mbundle body
-sendEmailDoc f t s cc scmd mbundle body =
-  if have_sendmail || scmd /= "" then do
+sendEmailDoc f t s cc scmd mbundle body = do
+  use_sendmail <- have_sendmail
+  if use_sendmail || scmd /= "" then do
     withOpenTemp $ \(h,fn) -> do
       generateEmail h f t s cc body
       hClose h
@@ -424,7 +387,8 @@ sendEmailDoc f t s cc scmd mbundle body =
         when (r /= ExitSuccess) $ fail ("failed to send mail to: "
                                        ++ t ++ cc_list cc
                                        ++ "\nPerhaps sendmail is not configured.")
-  else if have_mapi then do
+#ifdef HAVE_MAPI
+   else do
      r <- withCString t $ \tp ->
            withCString f $ \fp ->
             withCString cc $ \ccp ->
@@ -437,7 +401,9 @@ sendEmailDoc f t s cc scmd mbundle body =
                withCString cfn $ \pcfn ->
                 c_send_email fp tp ccp sp nullPtr pcfn
      when (r /= 0) $ fail ("failed to send mail to: " ++ t)
-  else fail $ "no mail facility (sendmail or mapi) located at configure time!"
+#else
+   else fail $ "no mail facility (sendmail or mapi) located at configure time!"
+#endif
   where addressOnly a =
           case dropWhile (/= '<') a of
           ('<':a2) -> takeWhile (/= '>') a2
@@ -448,9 +414,10 @@ sendEmailDoc f t s cc scmd mbundle body =
 
 resendEmail :: String -> String -> B.ByteString -> IO ()
 resendEmail "" _ _ = return ()
-resendEmail t scmd body =
-  case (have_sendmail || scmd /= "", have_mapi) of
-   (True, _) -> do
+resendEmail t scmd body = do
+  use_sendmail <- have_sendmail
+  if use_sendmail || scmd /= ""
+   then do
     withOpenTemp $ \(h,fn) -> do
      hPutStrLn h $ "To: "++ t
      hPutStrLn h $ find_from (linesPS body)
@@ -460,8 +427,12 @@ resendEmail t scmd body =
      let ftable = [('t',t)]
      r <-  execSendmail ftable scmd fn
      when (r /= ExitSuccess) $ fail ("failed to send mail to: " ++ t)
-   (_, True) -> fail "Don't know how to resend email with MAPI"
-   _ -> fail $ "no mail facility (sendmail or mapi) located at configure time (use the sendmail-command option)!"
+   else
+#ifdef HAVE_MAPI
+    fail "Don't know how to resend email with MAPI"
+#else
+    fail "no mail facility (sendmail or mapi) located at configure time (use the sendmail-command option)!"
+#endif
   where br            = BC.pack "\r"
         darcsurl      = BC.pack "DarcsURL:"
         content       = BC.pack "Content-"
@@ -483,8 +454,9 @@ resendEmail t scmd body =
 
 execSendmail :: [(Char,String)] -> String -> String -> IO ExitCode
 execSendmail ftable scmd fn =
-  if scmd == "" then
-     exec sendmail_path ["-i", "-t"] (File fn, Null, AsIs)
+  if scmd == "" then do
+     cmd <- sendmail_path
+     exec cmd ["-i", "-t"] (File fn, Null, AsIs)
   else case parseCmd (addUrlencoded ftable) scmd of
          Right (arg0:opts, wantstdin) ->
            do let stdin = if wantstdin then File fn else Null
@@ -494,9 +466,6 @@ execSendmail ftable scmd fn =
 
 #ifdef HAVE_MAPI
 foreign import ccall "win32/send_email.h send_email" c_send_email
-#else
-c_send_email
-#endif
              :: CString -> {- sender -}
                 CString -> {- recipient -}
                 CString -> {- cc -}
@@ -504,8 +473,6 @@ c_send_email
                 CString -> {- body -}
                 CString -> {- path -}
                 IO Int
-#ifndef HAVE_MAPI
-c_send_email = impossible
 #endif
 
 execPSPipe :: String -> [String] -> B.ByteString -> IO B.ByteString
@@ -629,15 +596,6 @@ verifySSL goodkeys s = do
                        _ -> return Nothing
     where opensslPS = execPSPipe "openssl"
 
-can_execute :: (String,Int) -> String -> IO Bool
-can_execute (arg,expected_return_value) exe = do
- withTemp $ \junk -> do
-  ec <- system (unwords [exe,arg,">",junk])
-  case ec of
-    ExitSuccess | expected_return_value == 0 -> return True
-    ExitFailure r | r == expected_return_value -> return True
-    _ -> return False
-
 
 {-
   - This function returns number of colors supported by current terminal
@@ -645,38 +603,14 @@ can_execute (arg,expected_return_value) exe = do
   - Terminal type determined by TERM env. variable.
   -}
 getTermNColors :: IO Int
-
 #ifdef HAVE_TERMINFO
 getTermNColors = do
   t <- setupTermFromEnv
   return $ case getCapability t $ tiGetNum "colors" of
     Nothing -> (-1)
     Just x -> x
-
-#elif HAVE_CURSES
-
-foreign import ccall "tgetnum" c_tgetnum :: CString -> IO CInt
-foreign import ccall "tgetent" c_tgetent :: Ptr CChar -> CString -> IO CInt
-
-termioBufSize :: Int
-termioBufSize = 4096
-
-getTermNColors = if not use_color
-                 then return (-1)
-                 else do term <- getEnv "TERM"
-                         allocaBytes termioBufSize (getTermNColorsImpl term)
-                      `catch` \_ -> return (-1)
-
-getTermNColorsImpl :: String -> Ptr CChar -> IO Int
-getTermNColorsImpl term buf = do rc <- withCString term $
-                                       \termp -> c_tgetent buf termp
-                                 x <- if (rc /= 1) then return (-1)  else withCString "Co" $ \capap -> c_tgetnum capap
-                                 return $ fromIntegral x
-
 #else
-
 getTermNColors = return (-1)
-
 #endif
 
 viewDoc :: Doc -> IO ()
