@@ -31,7 +31,7 @@ module Darcs.Patch.Depends ( get_common_and_uncommon, get_tags_right,
                ) where
 import Data.List ( delete, intersect )
 import Control.Monad ( liftM2 )
-import Control.Monad.Error (Error(..), MonadError(..))
+import Control.Monad.Error ( Error(..) )
 
 import Darcs.Patch ( RepoPatch, Named, getdeps, commutex,
                      commuteFL,
@@ -298,7 +298,7 @@ get_extra = get_extra_aux (return $ unsafeCoerceP NilFL)
                          $$ human_friendly (info $ n2pia hpc)
       where ep = case hopefullyM hp of
                  Right p' -> return p'
-                 Left e -> throwError (MissingPatch (info hp) e)
+                 Left e -> Left (MissingPatch (info hp) e)
 
 missingPatchError :: MissingPatch -> a
 missingPatchError (MissingPatch pinfo e) =
