@@ -27,11 +27,11 @@ module Darcs.Repository ( Repository, ($-), maybeIdentifyRepository,
                           withRepository, withRepositoryDirectory, withGutsOf,
                           makePatchLazy, writePatchSet,
                     findRepository, amInRepository, amNotInRepository,
-                    slurp_pending, replacePristine, replacePristineFromSlurpy,
+                    slurp_pending, replacePristineFromSlurpy,
                     slurp_recorded, slurp_recorded_and_unrecorded,
                     withRecorded,
                     get_unrecorded, get_unrecorded_unsorted, get_unrecorded_no_look_for_adds,
-                    get_unrecorded_in_files,
+                    get_unrecorded_in_files, get_unrecorded_in_files_unsorted,
                     read_repo, sync_repo,
                     prefsUrl,
                     add_to_pending,
@@ -59,11 +59,11 @@ import Darcs.Repository.Internal
      maybeIdentifyRepository, identifyRepositoryFor,
      findRepository, amInRepository, amNotInRepository,
      makePatchLazy,
-     slurp_pending, replacePristine, replacePristineFromSlurpy,
+     slurp_pending, replacePristineFromSlurpy,
      slurp_recorded, slurp_recorded_and_unrecorded,
      withRecorded,
      get_unrecorded, get_unrecorded_unsorted, get_unrecorded_no_look_for_adds,
-     get_unrecorded_in_files,
+     get_unrecorded_in_files, get_unrecorded_in_files_unsorted,
      read_repo, sync_repo,
      prefsUrl, checkPristineAgainstSlurpy,
      add_to_pending,
@@ -105,7 +105,7 @@ import Darcs.Repository.Prefs ( write_default_prefs )
 import Darcs.Repository.Pristine ( createPristine, flagsToPristine )
 import Darcs.Patch.Depends ( get_patches_beyond_tag )
 import Darcs.SlurpDirectory ( empty_slurpy )
-import Darcs.Utils ( withCurrentDirectory, catchall, promptYorn )
+import Darcs.Utils ( withCurrentDirectory, catchall, promptYorn, prettyError )
 import Darcs.External ( copyFileOrUrl, Cachable(..) )
 import Progress ( debugMessage, tediousSize,
                         beginTedious, endTedious, progress )
@@ -209,7 +209,7 @@ copyPartialRepository fromrepository@(Repo _ opts _ _) = do
            FlippedSeal ps <- return $ get_patches_beyond_tag pi_ch local_patches
            let needed_patches = reverseRL $ concatRL ps
            apply opts ch `catch`
-                             \e -> fail ("Bad checkpoint!\n" ++ show e)
+                             \e -> fail ("Bad checkpoint!\n" ++ prettyError e)
            apply_patches opts needed_patches
            debugMessage "Writing the pristine"
            pristineFromWorking torepository
