@@ -10,7 +10,8 @@ module Darcs.Repository.Cache (
                    unionCaches, cleanCaches, cleanCachesWithHint,
                    fetchFileUsingCache, speculateFileUsingCache, writeFileUsingCache,
                    findFileMtimeUsingCache, setFileMtimeUsingCache, peekInCache,
-                   repo2cache
+                   repo2cache,
+                   writable, isthisrepo, hashedFilePath, allHashedDirs
                  ) where
 
 import Control.Monad ( liftM, when, guard )
@@ -48,6 +49,9 @@ hashedDir HashedPristineDir = "pristine.hashed"
 hashedDir HashedPatchesDir = "patches"
 hashedDir HashedInventoriesDir = "inventories"
 
+allHashedDirs :: [HashedDir]
+allHashedDirs = [HashedPristineDir, HashedPatchesDir, HashedInventoriesDir]
+
 data WritableOrNot = Writable | NotWritable deriving ( Show )
 data CacheType = Repo | Directory deriving ( Eq, Show )
 data CacheLoc = Cache !CacheType !WritableOrNot !String
@@ -71,7 +75,7 @@ unionCaches (Ca a) (Ca b) = Ca (nub (a++b))
 repo2cache :: String -> Cache
 repo2cache r = Ca [Cache Repo NotWritable r]
 
--- This function computes the cache hash (i.e. filename) of a packed string.
+-- | 'cacheHash' computes the cache hash (i.e. filename) of a packed string.
 cacheHash :: B.ByteString -> String
 cacheHash ps = case show (B.length ps) of
                  x | l > 10 -> sha256sum ps
@@ -116,11 +120,20 @@ writable :: CacheLoc -> Bool
 writable (Cache _ NotWritable _) = False
 writable (Cache _ Writable _) = True
 
+isthisrepo :: CacheLoc -> Bool
+isthisrepo (Cache Repo Writable _) = True
+isthisrepo _ = False
+
+-- | @hashedFilePath cachelocation subdir hash@ returns the physical filename of
+-- hash @hash@ in the @subdir@ section of @cachelocation@.
 hashedFilePath :: CacheLoc -> HashedDir -> String -> String
 hashedFilePath (Cache Directory _ d) s f = d ++ "/" ++ (hashedDir s) ++ "/" ++ f
 hashedFilePath (Cache Repo _ r) s f =
     r ++ "/"++darcsdir++"/" ++ (hashedDir s) ++ "/" ++ f
 
+-- | @peekInCache cache subdir hash@ tells whether @cache@ and
+-- contains an object with hash @hash@ in a writable position.
+-- Florent: why do we want it to be in a writable position?
 peekInCache :: Cache -> HashedDir -> String -> IO Bool
 peekInCache (Ca cache) subdir f = cacheHasIt cache `catchall` return False
     where cacheHasIt [] = return False
@@ -130,8 +143,9 @@ peekInCache (Ca cache) subdir f = cacheHasIt cache `catchall` return False
                                                    else cacheHasIt cs
           fn c = hashedFilePath c subdir f
 
--- |Note that the file is likely to be useful soon: pipelined downloads will
--- add it to the (low-priority) queue, for the rest it is a noop.
+-- | @speculateFileUsingCache cache subdirectory name@ takes note that
+-- the file @name@ is likely to be useful soon: pipelined downloads
+-- will add it to the (low-priority) queue, for the rest it is a noop.
 speculateFileUsingCache :: Cache -> HashedDir -> String -> IO ()
 speculateFileUsingCache c sd h = do debugMessage $ "Speculating on "++h
                                     copyFileUsingCache OnlySpeculate c sd h
@@ -220,10 +234,16 @@ createCache (Cache Directory _ d) subdir =
     createDirectoryIfMissing True (d ++ "/" ++ (hashedDir subdir))
 createCache _ _ = return ()
 
+-- | @write compression filename content@ writes @content@ to the file @filename@ according
+-- to the policy given by @compression@.
 write :: Compression -> String -> B.ByteString -> IO ()
 write NoCompression = writeAtomicFilePS
 write GzipCompression = gzWriteAtomicFilePS
 
+-- | @writeFileUsingCache cache compression subdir contents@ write the string @contents@ to
+-- the directory subdir, except if it is already in the cache, in which case it is a noop.
+-- Warning (?) this means that in case of a hash collision, writing using writeFileUsingCache is
+-- a noop. The returned value is the filename that was given to the string.
 writeFileUsingCache :: Cache -> Compression -> HashedDir -> B.ByteString -> IO String
 writeFileUsingCache (Ca cache) compr subdir ps =
     (fetchFileUsingCachePrivate LocalOnly (Ca cache) subdir hash >> return hash) `catchall`
