@@ -22,7 +22,7 @@
 module Darcs.Repository.HashedRepo ( revert_tentative_changes, finalize_tentative_changes,
                                      slurp_pristine, sync_repo, clean_pristine,
                                      copy_pristine, copy_partials_pristine, pristine_from_working,
-                                     apply_to_tentative_pristine, replacePristine,
+                                     apply_to_tentative_pristine,
                                      replacePristineFromSlurpy,
                                      add_to_tentative_inventory, remove_from_tentative_inventory,
                                      read_repo, read_tentative_repo, write_and_read_patch,
@@ -208,15 +208,16 @@ read_repo_private repo d iname =
 write_and_read_patch :: RepoPatch p => Cache -> Compression -> PatchInfoAnd p C(x y)
                      -> IO (PatchInfoAnd p C(x y))
 write_and_read_patch c compr p = do (i,h) <- write_patch_if_necesary c compr p
-                                    Sealed x <- createHashed h (parse i)
-                                    return $ patchInfoAndPatch i $ unsafeCoerceP x
-    where parse i h = do debugMessage ("Reading patch file: "++ show (human_friendly i))
+                                    unsafeInterleaveIO $ readp h i
+    where parse i h = do debugMessage ("Rereading patch file: "++ show (human_friendly i))
                          (fn,ps) <- fetchFileUsingCache c HashedPatchesDir h
                          case readPatch ps of
                            Just (x,_) -> return x
                            Nothing -> fail $ unlines ["Couldn't parse patch file "++fn,
                                                       "which is",
                                                       renderString $ human_friendly i]
+          readp h i = do Sealed x <- createHashed h (parse i)
+                         return $ patchInfoAndPatch i $ unsafeCoerceP x
 
 write_tentative_inventory :: RepoPatch p => Cache -> Compression -> PatchSet p C(x) -> IO ()
 write_tentative_inventory c compr = write_either_inventory c compr "tentative_hashed_inventory"
@@ -262,9 +263,11 @@ write_inventory_private k c compr (x:<:xs) =
 write_patch_if_necesary :: RepoPatch p => Cache -> Compression
                         -> PatchInfoAnd p C(x y) -> IO (PatchInfo, String)
 write_patch_if_necesary c compr hp =
-    case extractHash hp of
-    Right h -> return (info hp, h)
-    Left p -> fmap (\h -> (info hp, h)) $ writeHashFile c compr HashedPatchesDir $ showPatch p
+    seq infohp $ case extractHash hp of
+                   Right h -> return (infohp, h)
+                   Left p -> (\h -> (infohp, h)) `fmap`
+                             writeHashFile c compr HashedPatchesDir (showPatch p)
+    where infohp = info hp
 
 pihash :: (PatchInfo,String) -> Doc
 pihash (pinf,hash) = showPatchInfo pinf $$ text ("hash: " ++ hash ++ "\n")
@@ -336,11 +339,9 @@ slurp_pristine_private c compr inv = case inv2pris inv of
                                       | otherwise -> slurpHashedPristine c compr h
 
 pristine_from_working :: Cache -> Compression -> IO ()
-pristine_from_working c compr = replacePristine c compr "."
-
-replacePristine :: Cache -> Compression -> FilePath -> IO ()
-replacePristine c compr d = do s <- slurp_all_but_darcs d
-                               replacePristineFromSlurpy c compr s
+pristine_from_working c compr = do
+  s <- slurp_all_but_darcs "."
+  replacePristineFromSlurpy c compr s
 
 replacePristineFromSlurpy :: Cache -> Compression -> Slurpy -> IO ()
 replacePristineFromSlurpy c compr s = do 
