@@ -26,10 +26,12 @@ module Darcs.Repository.Internal ( Repository(..), RepoType(..), RIO(unsafeUnRIO
                     findRepository, amInRepository, amNotInRepository,
                     slurp_pending, pristineFromWorking, revertRepositoryChanges,
                     slurp_recorded, slurp_recorded_and_unrecorded,
+                    read_pending, announce_merge_conflicts, setTentativePending,
+                    check_unrecorded_conflicts,
                     withRecorded,
                     checkPristineAgainstSlurpy,
                     get_unrecorded, get_unrecorded_unsorted, get_unrecorded_no_look_for_adds,
-                    get_unrecorded_in_files,
+                    get_unrecorded_in_files, get_unrecorded_in_files_unsorted,
                     read_repo, sync_repo,
                     prefsUrl, makePatchLazy,
                     add_to_pending,
@@ -42,7 +44,7 @@ module Darcs.Repository.Internal ( Repository(..), RepoType(..), RIO(unsafeUnRIO
                     unrevertUrl,
                     applyToWorking, patchSetToPatches,
                     createPristineDirectoryTree, createPartialsPristineDirectoryTree,
-                    replacePristine, replacePristineFromSlurpy,
+                    replacePristineFromSlurpy,
                     optimizeInventory, cleanRepository,
                     getMarkedupFile,
                     PatchSet, SealedPatchSet,
@@ -64,8 +66,8 @@ import Darcs.Repository.Pristine ( identifyPristine, nopristine,
                                    easyCreatePristineDirectoryTree, slurpPristine, syncPristine,
                                    easyCreatePartialsPristineDirectoryTree,
                                    createPristineFromWorking )
-import qualified Darcs.Repository.Pristine as Pristine ( replacePristine,
-                                                         replacePristineFromSlurpy )
+import qualified Darcs.Repository.Pristine as Pristine ( replacePristineFromSlurpy )
+                                                         
 import Data.List ( (\\) )
 import Darcs.SignalHandler ( withSignalsBlocked )
 import Darcs.Repository.Format ( RepoFormat, RepoProperty( Darcs2, HashedInventory ),
@@ -100,7 +102,7 @@ import qualified Darcs.Repository.HashedRepo as HashedRepo
                               write_tentative_inventory, write_and_read_patch,
                               add_to_tentative_inventory,
                               read_repo, read_tentative_repo, clean_pristine,
-                              replacePristine, replacePristineFromSlurpy,
+                              replacePristineFromSlurpy,
                               slurp_all_but_darcs )
 import qualified Darcs.Repository.DarcsRepo as DarcsRepo
 import Darcs.Flags ( DarcsFlag(AnyOrder, Boring, LookForAdds, Verbose, Quiet,
@@ -394,11 +396,16 @@ get_unrecorded_no_look_for_adds :: RepoPatch p => Repository p C(r u t) -> [File
 get_unrecorded_no_look_for_adds r paths = get_unrecorded_private (filter (/= LookForAdds)) r paths 
 
 get_unrecorded_unsorted :: RepoPatch p => Repository p C(r u t) -> IO (FL Prim C(r u))
-get_unrecorded_unsorted r = get_unrecorded_private (AnyOrder:) r []
+get_unrecorded_unsorted r = get_unrecorded_in_files_unsorted r []
 
 get_unrecorded :: RepoPatch p => Repository p C(r u t) -> IO (FL Prim C(r u))
 get_unrecorded r = get_unrecorded_private id r []
 
+-- | Gets the unrecorded changes in the given paths in the current repository,
+--   without sorting them for presentation to the user
+get_unrecorded_in_files_unsorted :: RepoPatch p => Repository p C(r u t) -> [FileName] -> IO (FL Prim C(r u))
+get_unrecorded_in_files_unsorted = get_unrecorded_private (AnyOrder:)
+
 -- | Gets the unrecorded changes in the given paths in the current repository.
 get_unrecorded_in_files :: RepoPatch p => Repository p C(r u t) -> [FileName] -> IO (FL Prim C(r u))
 get_unrecorded_in_files = get_unrecorded_private id 
@@ -923,6 +930,17 @@ remove_from_unrevert_context repository ps = do
                         where isr = isJust $ hopefullyM x
                  _ -> unrevert_impossible unrevert_loc
 
+-- | Writes out a fresh copy of the inventory that minimizes the
+-- amount of inventory that need be downloaded when people pull from
+-- the repository.
+--
+-- Specifically, it breaks up the inventory on the most recent tag.
+-- This speeds up most commands when run remotely, both because a
+-- smaller file needs to be transfered (only the most recent
+-- inventory).  It also gives a guarantee that all the patches prior
+-- to a given tag are included in that tag, so less commutation and
+-- history traversal is needed.  This latter issue can become very
+-- important in large repositories.
 optimizeInventory :: RepoPatch p => Repository p C(r u t) -> IO ()
 optimizeInventory repository@(Repo r opts rf (DarcsRepository _ c)) =
     do ps <- read_repo repository
@@ -939,11 +957,6 @@ cleanRepository repository@(Repo _ _ rf _) =
     HvsO { hashed = HashedRepo.clean_pristine repository,
            old = return () }
 
-replacePristine :: Repository p C(r u t) -> FilePath -> IO ()
-replacePristine (Repo r opts rf (DarcsRepository pris c)) d
-    | format_has HashedInventory rf = withCurrentDirectory r $ HashedRepo.replacePristine c (compression opts) d
-    | otherwise = withCurrentDirectory r $ Pristine.replacePristine d pris
-
 replacePristineFromSlurpy :: Repository p C(r u t) -> Slurpy -> IO ()
 replacePristineFromSlurpy (Repo r opts rf (DarcsRepository pris c)) s
     | format_has HashedInventory rf = withCurrentDirectory r $ HashedRepo.replacePristineFromSlurpy c (compression opts) s
@@ -991,7 +1004,11 @@ checkPristineAgainstSlurpy :: RepoPatch p => Repository p C(r u t) -> Slurpy ->
 checkPristineAgainstSlurpy repository@(Repo _ opts _ _) s2 =
     do s1 <- slurp_recorded repository
        ftf <- filetype_function
-       return $ nullFL $ unsafeDiff (LookForAdds:IgnoreTimes:opts) ftf s1 s2
+       -- The @$!@ is necessary because some code called from this function uses
+       -- unsafeInterleaveIO around functions that throw exceptions. If one used
+       -- @$@ instead of @$!@ here, those exceptions might not be caught by code
+       -- that runs this function inside a @catch@.
+       return $! nullFL $ unsafeDiff (LookForAdds:IgnoreTimes:opts) ftf s1 s2
 
 withTentative :: forall p a C(r u t). RepoPatch p =>
                  Repository p C(r u t) -> ((AbsolutePath -> IO a) -> IO a)
