@@ -80,14 +80,15 @@ applyAndFix c opts s_ r psin =
           aaf s NilFL = return ([], s, True)
           aaf s (p:>:ps) = do
             (s', mp') <- run_slurpy s $ applyAndTryToFix p
-            finishedOneIO k $ show $ human_friendly $ info p
+            let !infp = info p -- assure that 'p' can be garbage collected.
+            finishedOneIO k $ show $ human_friendly $ infp
             s'' <- syncSlurpy (update_slurpy r c opts) s'
             (ps', s''', restok) <- aaf s'' ps
             case mp' of
               Nothing -> return (ps', s''', restok)
               Just (e,pp) -> do putStrLn e
                                 p' <- makePatchLazy r pp
-                                return ((info p, p'):ps', s''', False)
+                                return ((infp, p'):ps', s''', False)
 
 data RepositoryConsistency p =
     RepositoryConsistent
@@ -135,7 +136,7 @@ replayRepository' repo opts = do
                   (ps, s_, ok) <- applyAndFix c opts s repo psin
                   debugMessage "Done fixing broken patches..."
                   return (s_, (reverseFL ps :<: NilRL), ok)
-  debugMessage "Checking pristine agains slurpy"
+  debugMessage "Checking pristine against slurpy"
   is_same <- checkPristineAgainstSlurpy repo s' `catchall` return False
   -- TODO is the latter condition needed? Does a broken patch imply pristine
   -- difference? Why, or why not?
