@@ -36,21 +36,21 @@ module Darcs.SelectChanges ( with_selected_changes',
                      ) where
 import System.IO
 import Data.List ( intersperse )
-import Data.Maybe ( catMaybes )
+import Data.Maybe ( catMaybes, isJust )
 import Data.Char ( toUpper )
 import Control.Monad ( when )
 import System.Exit ( exitWith, ExitCode(ExitSuccess) )
 
 import English ( Noun(..), englishNum  )
+import Darcs.Arguments ( showFriendly )
 import Darcs.Hopefully ( PatchInfoAnd, hopefully )
 import Darcs.Repository ( Repository, read_repo )
 import Darcs.Patch ( RepoPatch, Patchy, Prim, summary,
                      invert, list_touched_files,
                      commuteFL )
-import Darcs.Patch.Patchy ( showPatch )
 import qualified Darcs.Patch ( thing, things )
 import Darcs.Ordered ( FL(..), RL(..), (:>)(..),
-                       (+>+), lengthFL, concatRL, mapFL_FL,
+                       (+>+), lengthFL, lengthRL, concatRL, mapFL_FL,
                        spanFL, reverseFL, (+<+), mapFL,
                        unsafeCoerceP )
 import Darcs.Patch.Choices ( PatchChoices, patch_choices, patch_choices_tps,
@@ -66,11 +66,10 @@ import Darcs.Patch.Choices ( PatchChoices, patch_choices, patch_choices_tps,
                     )
 import Darcs.Patch.TouchesFiles ( deselect_not_touching, select_not_touching )
 import Darcs.PrintPatch ( printFriendly, printPatch, printPatchPager )
-import Darcs.SlurpDirectory ( Slurpy )
 import Darcs.Match ( have_nonrange_match, match_a_patch, match_a_patchread )
 import Darcs.Flags ( DarcsFlag( Summary, DontGrabDeps, Verbose, DontPromptForDependencies), isInteractive )
 import Darcs.Sealed ( FlippedSeal(..), flipSeal, seal2, unseal2 )
-import Darcs.Utils ( askUser, promptCharFancy, without_buffering )
+import Darcs.Utils ( askUser, promptCharFancy )
 import Printer ( prefix, putDocLn )
 #include "impossible.h"
 
@@ -81,7 +80,6 @@ type MatchCriterion p = FORALL(u v) WhichChanges -> [DarcsFlag] -> (p C(u v)) ->
 type WithPatches p a C(x y) =
         String              -- jobname
      -> [DarcsFlag]         -- opts
-     -> Slurpy              -- directory
      -> FL p C(x y)         -- patches to select among
      -> ((FL p :> FL p) C(x y) -> IO a) -- job
      -> IO a                -- result of running job
@@ -90,7 +88,6 @@ type WithPatches p a C(x y) =
 type WithPatchesToFiles p a C(x y) =
         String              -- jobname
      -> [DarcsFlag]         -- opts
-     -> Slurpy              -- directory
      -> [FilePath]          -- files
      -> FL p C(x y)         -- patches to select among
      -> ((FL p :> FL p) C(x y) -> IO a) -- job
@@ -136,39 +133,34 @@ with_selected_last_changes_reversed = wasc LastReversed iswanted
 
 -- | wasc and wasc_ are just shorthand for with_any_selected_changes
 wasc  :: Patchy p => WhichChanges -> MatchCriterion p -> WithPatches p a C(x y)
-wasc mwch crit j o s = wasc_ mwch crit j o s []
+wasc mwch crit j o = wasc_ mwch crit j o []
 wasc_ :: Patchy p => WhichChanges -> MatchCriterion p -> WithPatchesToFiles p a C(x y)
 wasc_ = with_any_selected_changes
 
 with_any_selected_changes :: Patchy p => WhichChanges -> MatchCriterion p -> WithPatchesToFiles p a C(x y)
-with_any_selected_changes Last crit jn opts s fs =
+with_any_selected_changes Last crit jn opts fs =
     with_any_selected_changes_last
         (patches_to_consider_last' fs opts crit)
-        crit jn opts s fs
-with_any_selected_changes First crit jn opts s fs =
+        crit jn opts fs
+with_any_selected_changes First crit jn opts fs =
     with_any_selected_changes_first
        (patches_to_consider_first' fs opts crit)
-       crit jn opts s fs
-with_any_selected_changes FirstReversed crit jn opts s fs =
+       crit jn opts fs
+with_any_selected_changes FirstReversed crit jn opts fs =
     with_any_selected_changes_first_reversed
        (patches_to_consider_first_reversed' fs opts crit)
-       crit jn opts s fs
-with_any_selected_changes LastReversed crit jn opts s fs =
+       crit jn opts fs
+with_any_selected_changes LastReversed crit jn opts fs =
     with_any_selected_changes_last_reversed
         (patches_to_consider_last_reversed' fs opts crit)
-        crit jn opts s fs
+        crit jn opts fs
 
 
-view_changes :: RepoPatch p => [DarcsFlag] -> Slurpy -> [FilePath] -> FL (PatchInfoAnd p) C(x y) -> IO ()
-view_changes opts _ fp ps =
-  case patches_to_consider_nothing' fp opts iswanted ps of
-  ps_to_consider :> _ -> vc ps_to_consider
- where
-       vc :: RepoPatch p => FL (PatchInfoAnd p) C(x y) -> IO ()
-       vc p = without_buffering $ do text_view opts ps_len 0 NilRL init_tps init_pc
-                                     return ()
-        where (init_pc, init_tps) = patch_choices_tps p
-              ps_len = lengthFL init_tps
+view_changes :: RepoPatch p => [DarcsFlag] -> FL (PatchInfoAnd p) C(x y) -> IO ()
+view_changes opts ps = do
+  text_view opts Nothing 0 NilRL init_tps init_pc
+  return ()
+    where (init_pc, init_tps) = patch_choices_tps ps
 
 data KeyPress a = KeyPress { kp     :: Char
                            , kpHelp :: String }
@@ -191,7 +183,7 @@ with_selected_patch_from_repo :: forall p C(r u t). RepoPatch p => String -> Rep
                               -> (FORALL(a) (FL (PatchInfoAnd p) :> PatchInfoAnd p) C(a r) -> IO ()) -> IO ()
 with_selected_patch_from_repo jn repository opts job = do
     p_s <- read_repo repository
-    sp <- without_buffering $ wspfr jn (match_a_patchread opts)
+    sp <- wspfr jn (match_a_patchread opts)
                               (concatRL p_s) NilFL
     case sp of
      Just (FlippedSeal (skipped :> selected)) -> job (skipped :> selected)
@@ -240,13 +232,12 @@ with_any_selected_changes_last :: forall p a C(x y). Patchy p
                                => (FL p C(x y) -> (FL p :> FL p) C(x y))
                                -> MatchCriterion p
                                -> WithPatchesToFiles p a C(x y)
-with_any_selected_changes_last p2c crit jobname opts _ _ ps job =
+with_any_selected_changes_last p2c crit jobname opts _ ps job =
  case p2c ps of
  ps_to_consider :> other_ps ->
          if not $ isInteractive opts
          then job $ ps_to_consider :> other_ps
-         else do pc <- without_buffering $
-                       tentatively_text_select "" jobname (Noun "patch") Last crit
+         else do pc <- tentatively_text_select "" jobname (Noun "patch") Last crit
                                               opts ps_len 0 NilRL init_tps init_pc
                  job $ selected_patches_last rejected_ps pc
          where rejected_ps = ps_to_consider
@@ -257,13 +248,12 @@ with_any_selected_changes_first :: forall p a C(x y). Patchy p
                                 => (FL p C(x y) -> (FL p :> FL p) C(x y))
                                 -> MatchCriterion p
                                 -> WithPatchesToFiles p a C(x y)
-with_any_selected_changes_first p2c crit jobname opts _ _ ps job =
+with_any_selected_changes_first p2c crit jobname opts _ ps job =
  case p2c ps of
  ps_to_consider :> other_ps ->
          if not $ isInteractive opts
          then job $ ps_to_consider :> other_ps
-         else do pc <- without_buffering $
-                       tentatively_text_select "" jobname (Noun "patch") First crit
+         else do pc <- tentatively_text_select "" jobname (Noun "patch") First crit
                                               opts ps_len 0 NilRL init_tps init_pc
                  job $ selected_patches_first rejected_ps pc
          where rejected_ps = other_ps
@@ -274,13 +264,12 @@ with_any_selected_changes_first_reversed :: forall p a C(x y). Patchy p
                                 => (FL p C(x y) -> (FL p :> FL p) C(y x))
                                 -> MatchCriterion p
                                 -> WithPatchesToFiles p a C(x y)
-with_any_selected_changes_first_reversed p2c crit jobname opts _ _ ps job =
+with_any_selected_changes_first_reversed p2c crit jobname opts _ ps job =
  case p2c ps of
  ps_to_consider :> other_ps ->
          if not $ isInteractive opts
          then job $ invert other_ps :> invert ps_to_consider
-         else do pc <- without_buffering $
-                       tentatively_text_select "" jobname (Noun "patch") FirstReversed crit
+         else do pc <- tentatively_text_select "" jobname (Noun "patch") FirstReversed crit
                                              opts ps_len 0 NilRL init_tps init_pc
                  job $ selected_patches_first_reversed rejected_ps pc
          where rejected_ps = ps_to_consider
@@ -291,13 +280,12 @@ with_any_selected_changes_last_reversed :: forall p a C(x y). Patchy p
                                 => (FL p C(x y) -> (FL p :> FL p) C(y x))
                                 -> MatchCriterion p
                                 -> WithPatchesToFiles p a C(x y)
-with_any_selected_changes_last_reversed p2c crit jobname opts _ _ ps job =
+with_any_selected_changes_last_reversed p2c crit jobname opts _ ps job =
  case p2c ps of
  ps_to_consider :> other_ps ->
          if not $ isInteractive opts
          then job $ invert other_ps :> invert ps_to_consider
-         else do pc <- without_buffering $
-                       tentatively_text_select "" jobname (Noun "patch") LastReversed crit
+         else do pc <- tentatively_text_select "" jobname (Noun "patch") LastReversed crit
                                              opts ps_len 0 NilRL init_tps init_pc
                  job $ selected_patches_last_reversed rejected_ps pc
          where rejected_ps = other_ps
@@ -382,25 +370,6 @@ patches_to_consider_last_reversed' fs opts crit ps =
     else tp_patches $ separate_first_middle_from_last $ deselect_not_touching fs
                      $ deselect_unwanted $ patch_choices $ invert ps
 
-patches_to_consider_nothing' :: RepoPatch p
-                     => [FilePath]  -- ^ files
-                     -> [DarcsFlag] -- ^ opts
-                     -> MatchCriterion (PatchInfoAnd p)
-                     -> FL (PatchInfoAnd p) C(x y) -- ^ patches
-                     -> (FL (PatchInfoAnd p) :> FL (PatchInfoAnd p)) C(x y)
-patches_to_consider_nothing' fs opts crit ps =
-  let deselect_unwanted pc =
-        if have_nonrange_match opts
-        then if DontGrabDeps `elem` opts
-             then force_matching_last (not.iswanted_) pc
-             else make_everything_later $ force_matching_first iswanted_ pc
-        else pc
-      iswanted_ = crit First opts . tp_patch
-  in if null fs && not (have_nonrange_match opts)
-     then ps :> NilFL
-     else tp_patches $ separate_first_middle_from_last $ deselect_not_touching fs
-                     $ deselect_unwanted $ patch_choices ps
-
 -- | Returns the results of a patch selection user interaction
 selected_patches_last :: Patchy p => FL p C(x y) -> PatchChoices p C(y z)
                       -> (FL p :> FL p) C(x z)
@@ -490,7 +459,7 @@ text_select jn whichch crit opts n_max n
                                                 then map_patches last_chs
                                                 else map_patches first_chs
                           map_patches = mapFL (\a ->
-                                           showPatch `unseal2` (seal2 $ tp_patch a))
+                                           (showFriendly opts) `unseal2` (seal2 $ tp_patch a))
                       putStrLn $ "---- Already selected "++things++" ----"
                       mapM_ putDocLn $ selected
                       putStrLn $ "---- end of already selected "++things++" ----"
@@ -540,7 +509,7 @@ text_select jn whichch crit opts n_max n
                      _ -> exitWith $ ExitSuccess
             else return ()
 
-text_view :: forall p C(x y u r s). Patchy p => [DarcsFlag] -> Int -> Int
+text_view :: forall p C(x y u r s). Patchy p => [DarcsFlag] -> Maybe Int -> Int
             -> RL (TaggedPatch p) C(x y) -> FL (TaggedPatch p) C(y u) -> PatchChoices p C(r s)
             -> IO ((PatchChoices p) C(r s))
 text_view _ _ _ _ NilFL _ = return $ patch_choices $ unsafeCoerceP NilFL --return pc
@@ -575,13 +544,14 @@ text_view opts n_max n
         options_nav =
           [ KeyPress 'q' ("quit view changes")
           , KeyPress 'k' "back up to previous patch"
-          , KeyPress 'j' "skip to next patch" ]
+          , KeyPress 'j' "skip to next patch"
+          , KeyPress 'c' "count total patch number" ]
         options = [ options_yn ]
                   ++ [ options_view ++
                        if Summary `elem` opts then [] else options_summary ]
                   ++ [ options_nav ]
         prompt = "Shall I view this patch? "
-               ++ "(" ++ show (n+1) ++ "/" ++ show n_max ++ ")"
+               ++ "(" ++ show (n+1) ++ "/" ++ maybe "?" show n_max ++ ")"
         repeat_this :: IO ((PatchChoices p) C(r s))
         repeat_this = do
           yorn <- promptCharFancy prompt (keysFor options) (Just 'n') "?h"
@@ -596,9 +566,11 @@ text_view opts n_max n
             'k' -> prev_patch
             'j' -> next_patch
             'c' -> text_view opts
-                       n_max n tps_done tps_todo pc
+                       count_n_max n tps_done tps_todo pc
             _   -> do putStrLn $ helpFor "view changes" options
                       repeat_this
+        count_n_max | isJust n_max = n_max
+                    | otherwise    = Just $ lengthFL tps_todo + lengthRL tps_done
 tentatively_text_select :: Patchy p => String -> String -> Noun -> WhichChanges
                         -> MatchCriterion p -> [DarcsFlag]
                         -> Int -> Int -> RL (TaggedPatch p) C(x y) -> FL (TaggedPatch p) C(y z)
