@@ -672,9 +672,6 @@ slurp_sync_size :: Int
 slurp_sync_size = 100 * 1000000
 
 syncSlurpy :: (Slurpy -> IO Slurpy) -> Slurpy -> IO Slurpy
-syncSlurpy put s = if (unsyncedSlurpySize s > slurp_sync_size)
-                       then do
-                          s' <- put s
-                          return s'
-                       else do
-                          return s
+syncSlurpy put s = if unsyncedSlurpySize s > slurp_sync_size
+                   then put s
+                   else return s
