@@ -33,6 +33,7 @@ import System.IO.Unsafe ( unsafePerformIO )
 import Data.Char ( toUpper, isDigit )
 import Data.Maybe ( fromMaybe )
 import Control.Monad ( liftM, liftM2 )
+import qualified Data.ByteString.Char8 as B
 
 type TimeInterval = (Maybe CalendarTime, Maybe CalendarTime)
 
@@ -73,18 +74,20 @@ readDate tz d =
 --   understood are those of 'showIsoDateTime' and 'date_time'
 parseDate :: Int -> String -> Either ParseError MCalendarTime
 parseDate tz d =
-              if length d >= 14 && and (map isDigit $ take 14 d)
+              if length d >= 14 && B.all isDigit bd
               then Right $ toMCalendarTime $
-                   CalendarTime (read $ take 4 d)
-                                (toEnum $ (+ (-1)) $ read $ take 2 $ drop 4 d)
-                                (read $ take 2 $ drop 6 d) -- Day
-                                (read $ take 2 $ drop 8 d) -- Hour
-                                (read $ take 2 $ drop 10 d) -- Minute
-                                (read $ take 2 $ drop 12 d) -- Second
+                   CalendarTime (readI $ B.take 4 bd)
+                                (toEnum $ (+ (-1)) $ readI $ B.take 2 $ B.drop 4 bd)
+                                (readI $ B.take 2 $ B.drop 6 bd) -- Day
+                                (readI $ B.take 2 $ B.drop 8 bd) -- Hour
+                                (readI $ B.take 2 $ B.drop 10 bd) -- Minute
+                                (readI $ B.take 2 $ B.drop 12 bd) -- Second
                                 0 Sunday 0 -- Picosecond, weekday and day of year unknown
                                 "GMT" 0 False
               else let dt = do { x <- date_time tz; eof; return x }
                    in parse dt "" d
+  where bd = B.pack (take 14 d)
+        readI s = fst $ fromMaybe (error "parseDate: invalid date") (B.readInt s)
 
 -- | Display a 'CalendarTime' in the ISO 8601 format without any
 --   separators, e.g. 20080825142503
@@ -472,9 +475,8 @@ second           = two_digits
 --
 --   * Universal timezones: UTC, UT
 --
---   * Some American timezones: EST, EDT, CST, CDT, MST, MDT, PST, PDT
---
---   * Some European timezones: UT, GMT, CEST, EEST
+--   * Zones from GNU coreutils/lib/getdate.y, less half-hour ones --
+--     sorry Newfies.
 --
 --   * any sequence of alphabetic characters (WARNING! treated as 0!)
 zone            :: CharParser a Int
@@ -484,6 +486,16 @@ zone             = choice
                        , mkZone "UTC"  0
                        , mkZone "UT"  0
                        , mkZone "GMT" 0
+                       , mkZone "WET" 0
+                       , mkZone "WEST" 1
+                       , mkZone "BST" 1
+                       , mkZone "ART" (-3)
+                       , mkZone "BRT" (-3)
+                       , mkZone "BRST" (-2)
+                       , mkZone "AST" (-4)
+                       , mkZone "ADT" (-3)
+                       , mkZone "CLT" (-4)
+                       , mkZone "CLST" (-3)
                        , mkZone "EST" (-5)
                        , mkZone "EDT" (-4)
                        , mkZone "CST" (-6)
@@ -492,8 +504,32 @@ zone             = choice
                        , mkZone "MDT" (-6)
                        , mkZone "PST" (-8)
                        , mkZone "PDT" (-7)
+                       , mkZone "AKST" (-9)
+                       , mkZone "AKDT" (-8)
+                       , mkZone "HST" (-10)
+                       , mkZone "HAST" (-10)
+                       , mkZone "HADT" (-9)
+                       , mkZone "SST" (-12)
+                       , mkZone "WAT" 1
+                       , mkZone "CET" 1
                        , mkZone "CEST" 2
+                       , mkZone "MET" 1
+                       , mkZone "MEZ" 1
+                       , mkZone "MEST" 2
+                       , mkZone "MESZ" 2
+                       , mkZone "EET" 2
                        , mkZone "EEST" 3
+                       , mkZone "CAT" 2
+                       , mkZone "SAST" 2
+                       , mkZone "EAT" 3
+                       , mkZone "MSK" 3
+                       , mkZone "MSD" 4
+                       , mkZone "SGT" 8
+                       , mkZone "KST" 9
+                       , mkZone "JST" 9
+                       , mkZone "GST" 10
+                       , mkZone "NZST" 12
+                       , mkZone "NZDT" 13
                          -- if we don't understand it, just give a GMT answer...
                        , do { manyTill (oneOf $ ['a'..'z']++['A'..'Z']++[' '])
                                        (lookAhead space_digit);
