@@ -59,34 +59,6 @@ int has_funky_char(const char *s, int len)
 
 }
 
-// mmapping...
-
-#ifdef _WIN32
-
-/* I have no idea if this works or not, and it is very tied to the usage
- * of mmap in FastPackedString. Most arguments are ignored...
- */
-
-char *my_mmap(size_t length, int fd)
-{
-  exit(1); /* mmap is not implemented on Windows */
-}
-
-int munmap(void *start, size_t length)
-{
-    UnmapViewOfFile(start);
-}
-
-#else
-
-char *my_mmap(size_t len, int fd) {
-  void *maybeok = mmap(NULL, len, PROT_READ, MAP_SHARED, fd, 0);
-  if (maybeok == MAP_FAILED) return NULL;
-  else return (char *)maybeok;
-}
-
-#endif
-
 // ForeignPtr debugging stuff...
 
 static int num_alloced = 0;
