@@ -1,10 +1,9 @@
 #!/usr/bin/env bash
 set -ev
 
-rm -rf temp
-mkdir temp
-echo temp
-cd temp
+rm -rf temp-$$
+mkdir temp-$$
+cd temp-$$
 set -e
 darcs initialize
 echo text > afile.txt
@@ -13,7 +12,7 @@ darcs record --author me --all --no-test --patch-name init
 darcs diff
 darcs diff -p . --store > diffinmem
 darcs diff -p . > diffondisk
-
 diff diffinmem diffondisk
+cd ..
 
-rm -rf temp
+rm -rf temp-$$
