@@ -23,9 +23,20 @@ lookfor () {
     rm -f "$RESULT"
 }
 
-
+# On 2009-02-12 Petr Rockai explained this on darcs-users:
+# The problem with Prelude readFile is that it's based on hGetContents, which
+# is lazy by definition. This also means that unless you force consumption of
+# the produced list, it will keep an fd open for the file, possibly
+# indefinitely.  This is called a fd leak. Other than being annoying and if done
+# often, leading to fd exhaustion and failure to open any new files (which is
+# usually fatal), it also prevents the file to be unlinked (deleted) on win32.
+#
+# On the other hand, *strict* bytestring version of readFile will read the whole
+# file into a contiguous buffer, *close the fd* and return. This is perfectly
+# safe with regards to fd leaks. Btw., this is *not* the case with lazy
+# bytestring variant of readFile, so that one is unsafe.
 lookfor readFile \
-        "readFile doesn't ensure the file is closed before it is deleted!" \
+        "Prelude.readFile doesn't ensure the file is closed before it is deleted!\nConsider import Data.ByteString.Char8 as B (readFile), B.readFile instead." \
         B # importing readFile from Data.ByteString as B, is allowed
 
 lookfor hGetContents \
