@@ -117,8 +117,7 @@ grep 'already exists' out
 
 touch abs_path.t
 darcs add abs_path.t
-IFS=""
-REPO_ABS=`hspwd`
+REPO_ABS=`pwd`
 darcs mv "$REPO_ABS/abs_path.t" abs_path_new.t
 darcs mv abs_path_new.t "$REPO_ABS/abs_path.t"
 
