@@ -1,8 +1,6 @@
 #!/usr/bin/env bash
-set -ev
 
-## I would use the builtin !, but that has the wrong semantics.
-not () { "$@" && exit 1 || :; }
+. lib
 
 rm -rf temp1
 mkdir temp1
