@@ -4,10 +4,7 @@
 
 . lib
 
-# setting IFS is an ugly hack for Cygwin
-# so that the hspwd backtick
-IFS=''
-DIR=`hspwd`
+DIR="`pwd`"
 
 rm -rf temp1 temp2
 mkdir temp1
@@ -43,8 +40,8 @@ cd temp1
 # Before trying to pull from self, defaultrepo does not exist
 test ! -e _darcs/prefs/defaultrepo
 # return special message when you try to push to yourself
-not darcs push -a ${DIR}/temp1 2> log
-grep -i "can't push to current repository!" log
+not darcs push -a "$DIR/temp1" 2> log
+grep -i "cannot push from repository to itself" log
 # and don't update the default repo to be the current dir
 test ! -e _darcs/prefs/defaultrepo
 cd ..
