@@ -31,13 +31,10 @@ touch b.t
 darcs record  -am foo a.t b.t > log
 grep -i 'non existent files or directories: "a.t"' log
 
-# record works with absolute paths
-# hack for Cygwin and hspwd
-IFS=''
-DIR=`hspwd`
+DIR="`pwd`"
 touch date.t
 darcs add date.t
-darcs record -a -m foo ${DIR}/date.t | grep -i 'finished recording'
+darcs record -a -m foo "$DIR/date.t" | grep -i 'finished recording'
 
 # issue396 - record -l ""
 touch 'notnull.t'
